@@ -85,7 +85,7 @@ class Batch(object):
         '''
         partition = lambda x: float(x) / 100.0 * len(self.minions)
         try:
-            if '%' in self.opts['batch']:
+            if isinstance(self.opts['batch'], six.string_types) and '%' in self.opts['batch']:
                 res = partition(float(self.opts['batch'].strip('%')))
                 if res < 1:
                     return int(math.ceil(res))
