@@ -49,14 +49,16 @@ Execution module for Amazon Route53 written against Boto 3
 #pylint: disable=E0602,W0106
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import time
+import re
 
 # Import Salt libs
-import salt.utils.boto3
 import salt.utils.compat
-from salt.exceptions import SaltInvocationError
+import salt.utils.versions
+from salt.exceptions import SaltInvocationError, CommandExecutionError
+from salt.ext import six
 log = logging.getLogger(__name__)  # pylint: disable=W1699
 
 # Import third party libs
@@ -76,9 +78,7 @@ def __virtual__():
     Only load if boto libraries exist and if boto libraries are greater than
     a given version.
     '''
-    if not HAS_BOTO3:
-        return (False, 'The boto3_route53 module could not be loaded: boto3 libraries not found')
-    return True
+    return salt.utils.versions.check_boto_reqs()
 
 
 def __init__(opts):
@@ -101,7 +101,7 @@ def _collect_results(func, item, args, marker='Marker', nextmarker='NextMarker')
                 time.sleep(3)
                 tries -= 1
                 continue
-            log.error('Could not collect results from {0}(): {1}'.format(func, str(e)))
+            log.error('Could not collect results from %s(): %s', func, e)
             return []
         i = r.get(item, []) if item else r
         i.pop('ResponseMetadata', None) if isinstance(i, dict) else None
@@ -113,7 +113,7 @@ def _collect_results(func, item, args, marker='Marker', nextmarker='NextMarker')
 
 def _wait_for_sync(change, conn, tries=10, sleep=20):
     for retry in range(1, tries+1):
-        log.info('Getting route53 status (attempt {0})'.format(retry))
+        log.info('Getting route53 status (attempt %s)', retry)
         status = 'wait'
         try:
             status = conn.get_change(Id=change)['ChangeInfo']['Status']
@@ -176,8 +176,10 @@ def find_hosted_zone(Id=None, Name=None, PrivateZone=None,
     if PrivateZone is not None:
         ret = [m for m in ret if m['HostedZone']['Config']['PrivateZone'] is PrivateZone]
     if len(ret) > 1:
-        log.error('Request matched more than one Hosted Zone ({0}).  Refine your criteria and try '
-                  'again.'.format([z['HostedZone']['Id'] for z in ret]))
+        log.error(
+            'Request matched more than one Hosted Zone (%s). Refine your '
+            'criteria and try again.', [z['HostedZone']['Id'] for z in ret]
+        )
         ret = []
     return ret
 
@@ -243,7 +245,7 @@ def get_hosted_zones_by_domain(Name, region=None, key=None, keyid=None, profile=
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     zones = [z for z in _collect_results(conn.list_hosted_zones, 'HostedZones', {})
-            if z['Name'] == Name]
+            if z['Name'] == aws_encode(Name)]
     ret = []
     for z in zones:
         ret += get_hosted_zone(Id=z['Id'], region=region, key=key, keyid=keyid, profile=profile)
@@ -344,12 +346,15 @@ def create_hosted_zone(Name, VPCId=None, VPCName=None, VPCRegion=None, CallerRef
     '''
     if not Name.endswith('.'):
         raise SaltInvocationError('Domain must be fully-qualified, complete with trailing period.')
+    Name = aws_encode(Name)
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     deets = find_hosted_zone(Name=Name, PrivateZone=PrivateZone,
                              region=region, key=key, keyid=keyid, profile=profile)
     if deets:
-        log.info('Route 53 hosted zone {0} already exists.  You may want to pass e.g. '
-                 "'PrivateZone=True' or similar...".format(Name))
+        log.info(
+            'Route 53 hosted zone %s already exists. You may want to pass '
+            'e.g. \'PrivateZone=True\' or similar...', Name
+        )
         return None
     args = {
             'Name': Name,
@@ -373,8 +378,10 @@ def create_hosted_zone(Name, VPCId=None, VPCName=None, VPCRegion=None, CallerRef
             log.error('Private zone requested but no VPC matching given criteria found.')
             return None
         if len(vpcs) > 1:
-            log.error('Private zone requested but multiple VPCs matching given criteria found: '
-                      '{0}.'.format([v['id'] for v in vpcs]))
+            log.error(
+                'Private zone requested but multiple VPCs matching given '
+                'criteria found: %s.', [v['id'] for v in vpcs]
+            )
             return None
         vpc = vpcs[0]
         if VPCName:
@@ -398,9 +405,9 @@ def create_hosted_zone(Name, VPCId=None, VPCName=None, VPCRegion=None, CallerRef
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
-                retries -= 1
+                tries -= 1
                 continue
-            log.error('Failed to create hosted zone {0}: {1}'.format(Name, str(e)))
+            log.error('Failed to create hosted zone %s: %s', Name, e)
             return []
     return []
 
@@ -435,7 +442,7 @@ def update_hosted_zone_comment(Id=None, Name=None, Comment=None, PrivateZone=Non
                 'key': key, 'keyid': keyid, 'profile': profile}
         zone = find_hosted_zone(**args)
         if not zone:
-            log.error("Couldn't resolve domain name {0} to a hosted zone ID.".format(Name))
+            log.error("Couldn't resolve domain name %s to a hosted zone ID.", Name)
             return []
         Id = zone[0]['HostedZone']['Id']
     tries = 10
@@ -448,10 +455,10 @@ def update_hosted_zone_comment(Id=None, Name=None, Comment=None, PrivateZone=Non
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
-                retries -= 1
+                tries -= 1
                 continue
-            log.error('Failed to update comment on hosted zone {0}: {1}'.format(
-                      Name or Id, str(e)))
+            log.error('Failed to update comment on hosted zone %s: %s',
+                      Name or Id, e)
     return []
 
 
@@ -514,8 +521,10 @@ def associate_vpc_with_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
                 'key': key, 'keyid': keyid, 'profile': profile}
         zone = find_hosted_zone(**args)
         if not zone:
-            log.error("Couldn't resolve domain name {0} to a private hosted zone"
-                      'ID.'.format(Name))
+            log.error(
+                "Couldn't resolve domain name %s to a private hosted zone ID.",
+                Name
+            )
             return False
         HostedZoneId = zone[0]['HostedZone']['Id']
     vpcs = __salt__['boto_vpc.describe_vpcs'](vpc_id=VPCId, name=VPCName, region=region, key=key,
@@ -526,8 +535,8 @@ def associate_vpc_with_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
         log.error('No VPC matching the given criteria found.')
         return False
     if len(vpcs) > 1:
-        log.error('Multiple VPCs matching the given criteria found: {0}.'
-                  ''.format(', '.join([v['id'] for v in vpcs])))
+        log.error('Multiple VPCs matching the given criteria found: %s.',
+                  ', '.join([v['id'] for v in vpcs]))
         return False
     vpc = vpcs[0]
     if VPCName:
@@ -544,17 +553,21 @@ def associate_vpc_with_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
             r = conn.associate_vpc_with_hosted_zone(**args)
             return _wait_for_sync(r['ChangeInfo']['Id'], conn)
         except ClientError as e:
+            if e.response.get('Error', {}).get('Code') == 'ConflictingDomainExists':
+                log.debug('VPC Association already exists.')
+                # return True since the current state is the desired one
+                return True
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
-                retries -= 1
+                tries -= 1
                 continue
-            log.error('Failed to associate VPC {0} with hosted zone {1}: {2}'.format(
-                      VPCName or VPCId, Name or HostedZoneId, str(e)))
+            log.error('Failed to associate VPC %s with hosted zone %s: %s',
+                      VPCName or VPCId, Name or HostedZoneId, e)
     return False
 
 
-def diassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
+def disassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
                                      VPCName=None, VPCRegion=None, Comment=None,
                                      region=None, key=None, keyid=None, profile=None):
     '''
@@ -606,8 +619,7 @@ def diassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
                 'key': key, 'keyid': keyid, 'profile': profile}
         zone = find_hosted_zone(**args)
         if not zone:
-            log.error("Couldn't resolve domain name {0} to a private hosted zone"
-                      'ID.'.format(Name))
+            log.error("Couldn't resolve domain name %s to a private hosted zone ID.", Name)
             return False
         HostedZoneId = zone[0]['HostedZone']['Id']
     vpcs = __salt__['boto_vpc.describe_vpcs'](vpc_id=VPCId, name=VPCName, region=region, key=key,
@@ -618,8 +630,8 @@ def diassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
         log.error('No VPC matching the given criteria found.')
         return False
     if len(vpcs) > 1:
-        log.error('Multiple VPCs matching the given criteria found: {0}.'
-                  ''.format(', '.join([v['id'] for v in vpcs])))
+        log.error('Multiple VPCs matching the given criteria found: %s.',
+                  ', '.join([v['id'] for v in vpcs]))
         return False
     vpc = vpcs[0]
     if VPCName:
@@ -636,13 +648,17 @@ def diassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
             r = conn.disassociate_vpc_from_hosted_zone(**args)
             return _wait_for_sync(r['ChangeInfo']['Id'], conn)
         except ClientError as e:
+            if e.response.get('Error', {}).get('Code') == 'VPCAssociationNotFound':
+                log.debug('No VPC Association exists.')
+                # return True since the current state is the desired one
+                return True
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
-                retries -= 1
+                tries -= 1
                 continue
-            log.error('Failed to associate VPC {0} with hosted zone {1}: {2}'.format(
-                      VPCName or VPCId, Name or HostedZoneId, str(e)))
+            log.error('Failed to associate VPC %s with hosted zone %s: %s',
+                      VPCName or VPCId, Name or HostedZoneId, e)
     return False
 
 
@@ -680,7 +696,7 @@ def delete_hosted_zone(Id, region=None, key=None, keyid=None, profile=None):
         r = conn.delete_hosted_zone(Id=Id)
         return _wait_for_sync(r['ChangeInfo']['Id'], conn)
     except ClientError as e:
-        log.error('Failed to delete hosted zone {0}: {1}'.format(Id, str(e)))
+        log.error('Failed to delete hosted zone %s: %s', Id, e)
     return False
 
 
@@ -699,12 +715,102 @@ def delete_hosted_zone_by_domain(Name, PrivateZone=None, region=None, key=None,
     # name resolves to both, fail and require them to declare it explicitly.
     zone = find_hosted_zone(**args)
     if not zone:
-        log.error("Couldn't resolve domain name {0} to a hosted zone ID.".format(Name))
+        log.error("Couldn't resolve domain name %s to a hosted zone ID.", Name)
         return False
     Id = zone[0]['HostedZone']['Id']
     return delete_hosted_zone(Id=Id, region=region, key=key, keyid=keyid, profile=profile)
 
 
+def aws_encode(x):
+    '''
+    An implementation of the encoding required to suport AWS's domain name
+    rules defined here__:
+
+    .. __: http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DomainNameFormat.html
+
+    While AWS's documentation specifies individual ASCII characters which need
+    to be encoded, we instead just try to force the string to one of
+    escaped unicode or idna depending on whether there are non-ASCII characters
+    present.
+
+    This means that we support things like ドメイン.テスト as a domain name string.
+
+    More information about IDNA encoding in python is found here__:
+
+    .. __: https://pypi.org/project/idna
+
+    '''
+    ret = None
+    try:
+        x.encode('ascii')
+        ret = re.sub(r'\\x([a-f0-8]{2})',
+                      _hexReplace, x.encode('unicode_escape'))
+    except UnicodeEncodeError:
+        ret = x.encode('idna')
+    except Exception as e:
+        log.error("Couldn't encode %s using either 'unicode_escape' or 'idna' codecs", x)
+        raise CommandExecutionError(e)
+    log.debug('AWS-encoded result for %s: %s', x, ret)
+    return ret
+
+
+def _aws_encode_changebatch(o):
+    '''
+    helper method to process a change batch & encode the bits which need encoding.
+    '''
+    change_idx = 0
+    while change_idx < len(o['Changes']):
+        o['Changes'][change_idx]['ResourceRecordSet']['Name'] = aws_encode(o['Changes'][change_idx]['ResourceRecordSet']['Name'])
+        if 'ResourceRecords' in o['Changes'][change_idx]['ResourceRecordSet']:
+            rr_idx = 0
+            while rr_idx < len(o['Changes'][change_idx]['ResourceRecordSet']['ResourceRecords']):
+                o['Changes'][change_idx]['ResourceRecordSet']['ResourceRecords'][rr_idx]['Value'] = aws_encode(o['Changes'][change_idx]['ResourceRecordSet']['ResourceRecords'][rr_idx]['Value'])
+                rr_idx += 1
+        if 'AliasTarget' in o['Changes'][change_idx]['ResourceRecordSet']:
+            o['Changes'][change_idx]['ResourceRecordSet']['AliasTarget']['DNSName'] = aws_encode(o['Changes'][change_idx]['ResourceRecordSet']['AliasTarget']['DNSName'])
+        change_idx += 1
+    return o
+
+
+def _aws_decode(x):
+    '''
+    An implementation of the decoding required to suport AWS's domain name
+    rules defined here__:
+
+    .. __: http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DomainNameFormat.html
+
+    The important part is this:
+
+        If the domain name includes any characters other than a to z, 0 to 9, - (hyphen),
+        or _ (underscore), Route 53 API actions return the characters as escape codes.
+        This is true whether you specify the characters as characters or as escape
+        codes when you create the entity.
+        The Route 53 console displays the characters as characters, not as escape codes."
+
+        For a list of ASCII characters the corresponding octal codes, do an internet search on "ascii table".
+
+    We look for the existance of any escape codes which give us a clue that
+    we're received an escaped unicode string; or we assume it's idna encoded
+    and then decode as necessary.
+    '''
+    if '\\' in x:
+        return x.decode('unicode_escape')
+    return x.decode('idna')
+
+
+def _hexReplace(x):
+    '''
+    Converts a hex code to a base 16 int then the octal of it, minus the leading
+    zero.
+
+    This is necessary because x.encode('unicode_escape') automatically assumes
+    you want a hex string, which AWS will accept but doesn't result in what
+    you really want unless it's an octal escape sequence
+    '''
+    c = int(x.group(1), 16)
+    return '\\' + str(oct(c))[1:]
+
+
 def get_resource_records(HostedZoneId=None, Name=None, StartRecordName=None,
                          StartRecordType=None, PrivateZone=None,
                          region=None, key=None, keyid=None, profile=None):
@@ -736,7 +842,7 @@ def get_resource_records(HostedZoneId=None, Name=None, StartRecordName=None,
         args.update({'PrivateZone': PrivateZone}) if PrivateZone is not None else None
         zone = find_hosted_zone(**args)
         if not zone:
-            log.error("Couldn't resolve domain name {0} to a hosted zone ID.".format(Name))
+            log.error("Couldn't resolve domain name %s to a hosted zone ID.", Name)
             return []
         HostedZoneId = zone[0]['HostedZone']['Id']
 
@@ -750,7 +856,7 @@ def get_resource_records(HostedZoneId=None, Name=None, StartRecordName=None,
         if done:
             return ret
         args = {'HostedZoneId': HostedZoneId}
-        args.update({'StartRecordName': next_rr_name}) if next_rr_name else None
+        args.update({'StartRecordName': aws_encode(next_rr_name)}) if next_rr_name else None
         # Grrr, can't specify type unless name is set...  We'll do this via filtering later instead
         args.update({'StartRecordType': next_rr_type}) if next_rr_name and next_rr_type else None
         args.update({'StartRecordIdentifier': next_rr_id}) if next_rr_id else None
@@ -761,6 +867,18 @@ def get_resource_records(HostedZoneId=None, Name=None, StartRecordName=None,
             next_rr_type = r.get('NextRecordType')
             next_rr_id = r.get('NextRecordIdentifier')
             for rr in rrs:
+                rr['Name'] = _aws_decode(rr['Name'])
+                # now iterate over the ResourceRecords and replace any encoded
+                # value strings with the decoded versions
+                if 'ResourceRecords' in rr:
+                    x = 0
+                    while x < len(rr['ResourceRecords']):
+                        if 'Value' in rr['ResourceRecords'][x]:
+                            rr['ResourceRecords'][x]['Value'] = _aws_decode(rr['ResourceRecords'][x]['Value'])
+                        x += 1
+                # or if we are an AliasTarget then decode the DNSName
+                if 'AliasTarget' in rr:
+                    rr['AliasTarget']['DNSName'] = _aws_decode(rr['AliasTarget']['DNSName'])
                 if StartRecordName and rr['Name'] != StartRecordName:
                     done = True
                     break
@@ -787,9 +905,6 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
                                 PrivateZone=None, ChangeBatch=None,
                                 region=None, key=None, keyid=None, profile=None):
     '''
-    Ugh!!!  Not gonna try to reproduce and validatethis mess in here - just pass what we get to AWS
-    and let it decide if it's valid or not...
-
     See the `AWS Route53 API docs`__ as well as the `Boto3 documentation`__ for all the details...
 
     .. __: https://docs.aws.amazon.com/Route53/latest/APIReference/API_ChangeResourceRecordSets.html
@@ -799,41 +914,42 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
     parameters and combinations thereof are quite varied, so perusal of the above linked docs is
     highly recommended for any non-trival configurations.
 
-    .. code-block:: json
-    ChangeBatch={
-        'Comment': 'string',
-        'Changes': [
-            {
-                'Action': 'CREATE'|'DELETE'|'UPSERT',
-                'ResourceRecordSet': {
-                    'Name': 'string',
-                    'Type': 'SOA'|'A'|'TXT'|'NS'|'CNAME'|'MX'|'NAPTR'|'PTR'|'SRV'|'SPF'|'AAAA',
-                    'SetIdentifier': 'string',
-                    'Weight': 123,
-                    'Region': 'us-east-1'|'us-east-2'|'us-west-1'|'us-west-2'|'ca-central-1'|'eu-west-1'|'eu-west-2'|'eu-central-1'|'ap-southeast-1'|'ap-southeast-2'|'ap-northeast-1'|'ap-northeast-2'|'sa-east-1'|'cn-north-1'|'ap-south-1',
-                    'GeoLocation': {
-                        'ContinentCode': 'string',
-                        'CountryCode': 'string',
-                        'SubdivisionCode': 'string'
-                    },
-                    'Failover': 'PRIMARY'|'SECONDARY',
-                    'TTL': 123,
-                    'ResourceRecords': [
-                        {
-                            'Value': 'string'
+    .. code-block:: text
+
+        {
+            "Comment": "string",
+            "Changes": [
+                {
+                    "Action": "CREATE"|"DELETE"|"UPSERT",
+                    "ResourceRecordSet": {
+                        "Name": "string",
+                        "Type": "SOA"|"A"|"TXT"|"NS"|"CNAME"|"MX"|"NAPTR"|"PTR"|"SRV"|"SPF"|"AAAA",
+                        "SetIdentifier": "string",
+                        "Weight": 123,
+                        "Region": "us-east-1"|"us-east-2"|"us-west-1"|"us-west-2"|"ca-central-1"|"eu-west-1"|"eu-west-2"|"eu-central-1"|"ap-southeast-1"|"ap-southeast-2"|"ap-northeast-1"|"ap-northeast-2"|"sa-east-1"|"cn-north-1"|"ap-south-1",
+                        "GeoLocation": {
+                            "ContinentCode": "string",
+                            "CountryCode": "string",
+                            "SubdivisionCode": "string"
                         },
-                    ],
-                    'AliasTarget': {
-                        'HostedZoneId': 'string',
-                        'DNSName': 'string',
-                        'EvaluateTargetHealth': True|False
-                    },
-                    'HealthCheckId': 'string',
-                    'TrafficPolicyInstanceId': 'string'
-                }
-            },
-        ]
-    }
+                        "Failover": "PRIMARY"|"SECONDARY",
+                        "TTL": 123,
+                        "ResourceRecords": [
+                            {
+                                "Value": "string"
+                            },
+                        ],
+                        "AliasTarget": {
+                            "HostedZoneId": "string",
+                            "DNSName": "string",
+                            "EvaluateTargetHealth": True|False
+                        },
+                        "HealthCheckId": "string",
+                        "TrafficPolicyInstanceId": "string"
+                    }
+                },
+            ]
+        }
 
     CLI Example:
 
@@ -862,11 +978,12 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
         args.update({'PrivateZone': PrivateZone}) if PrivateZone is not None else None
         zone = find_hosted_zone(**args)
         if not zone:
-            log.error("Couldn't resolve domain name {0} to a hosted zone ID.".format(Name))
+            log.error("Couldn't resolve domain name %s to a hosted zone ID.", Name)
             return []
         HostedZoneId = zone[0]['HostedZone']['Id']
 
-    args = {'HostedZoneId': HostedZoneId, 'ChangeBatch': ChangeBatch}
+    args = {'HostedZoneId': HostedZoneId, 'ChangeBatch': _aws_encode_changebatch(ChangeBatch)}
+
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     tries = 20  # A bit more headroom
     while tries:
@@ -877,8 +994,9 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
-                retries -= 1
+                tries -= 1
                 continue
-            log.error('Failed to apply requested changes to the hosted zone {0}: {1}'.format(
-                    Name or HostedZoneId, str(e)))
+            log.error('Failed to apply requested changes to the hosted zone %s: %s',
+                    (Name or HostedZoneId), six.text_type(e))
+            raise e
     return False
