@@ -44,7 +44,6 @@ import salt.minion
 import salt.payload
 import salt.syspaths
 import salt.exceptions
-import salt.log.setup as log_setup
 import salt.defaults.exitcodes
 from salt.utils.odict import OrderedDict
 
@@ -130,7 +129,8 @@ class Schedule(object):
                            cleanup=None,
                            proxy=None,
                            standalone=False,
-                           utils=None):
+                           utils=None,
+                           _subprocess_list=None):
         self.opts = opts
         self.proxy = proxy
         self.functions = functions
@@ -158,6 +158,10 @@ class Schedule(object):
         if cleanup:
             for prefix in cleanup:
                 self.delete_job_prefix(prefix)
+        if _subprocess_list is None:
+            self._subprocess_list = salt.utils.process.SubprocessList()
+        else:
+            self._subprocess_list = _subprocess_list
 
     def __getnewargs__(self):
         return self.opts, self.functions, self.returners, self.intervals, None
@@ -192,9 +196,10 @@ class Schedule(object):
         if remove_hidden:
             _schedule = copy.deepcopy(schedule)
             for job in _schedule:
-                for item in _schedule[job]:
-                    if item.startswith('_'):
-                        del schedule[job][item]
+                if isinstance(_schedule[job], dict):
+                    for item in _schedule[job]:
+                        if item.startswith('_'):
+                            del schedule[job][item]
         return schedule
 
     def _check_max_running(self, func, data, opts, now):
@@ -239,7 +244,7 @@ class Schedule(object):
                             )
                             data['_skip_reason'] = 'maxrunning'
                             data['_skipped'] = True
-                            data['_skip_time'] = now
+                            data['_skipped_time'] = now
                             data['run'] = False
                             return data
         return data
@@ -654,14 +659,6 @@ class Schedule(object):
                 ret['jid']
             )
 
-        if multiprocessing_enabled and not salt.utils.platform.is_windows():
-            # Reconfigure multiprocessing logging after daemonizing
-            log_setup.setup_multiprocessing_logging()
-
-        if multiprocessing_enabled:
-            # Don't *BEFORE* to go into try to don't let it triple execute the finally section.
-            salt.utils.process.daemonize_if(self.opts)
-
         # TODO: Make it readable! Splt to funcs, remove nested try-except-finally sections.
         try:
 
@@ -1377,8 +1374,10 @@ class Schedule(object):
             # Clear these out between runs
             for item in ['_continue',
                          '_error',
+                         '_enabled',
                          '_skipped',
-                         '_skip_reason']:
+                         '_skip_reason',
+                         '_skipped_time']:
                 if item in data:
                     del data[item]
             run = False
@@ -1599,10 +1598,20 @@ class Schedule(object):
             if 'enabled' not in data:
                 data['enabled'] = self.enabled
 
+            # If globally disabled, disable the job
+            if not self.enabled:
+                data['enabled'] = self.enabled
+                data['_skip_reason'] = 'disabled'
+                data['_skipped_time'] = now
+                data['_skipped'] = True
+                run = False
+
             # Job is disabled, set run to False
             if 'enabled' in data and not data['enabled']:
-                log.debug('Job: %s is disabled', job_name)
+                data['_enabled'] = False
                 data['_skip_reason'] = 'disabled'
+                data['_skipped_time'] = now
+                data['_skipped'] = True
                 run = False
 
             miss_msg = ''
@@ -1697,10 +1706,13 @@ class Schedule(object):
                     # Reset current signals before starting the process in
                     # order not to inherit the current signal handlers
                     proc.start()
-                proc.join()
+                    proc.name = '{}-Schedule-{}'.format(proc.name, data['name'])
+                    self._subprocess_list.add(proc)
             else:
                 proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
                 proc.start()
+                proc.name = '{}-Schedule-{}'.format(proc.name, data['name'])
+                self._subprocess_list.add(proc)
         finally:
             if multiprocessing_enabled and salt.utils.platform.is_windows():
                 # Restore our function references.
@@ -1708,6 +1720,9 @@ class Schedule(object):
                 self.returners = returners
                 self.utils = utils
 
+    def cleanup_subprocesses(self):
+        self._subprocess_list.cleanup()
+
 
 def clean_proc_dir(opts):
 
